@@ -21,6 +21,7 @@ from ..account import Account
 from ..cache import Cache
 from ..group import Group
 from ..openorders import OpenOrders
+from ..perpmarketinfo import PerpMarketInfo
 from ..spotmarketinfo import SpotMarketInfo
 from ..tokenvalue import TokenValue
 
@@ -51,22 +52,31 @@ class SpotCollateralCalculator(CollateralCalculator):
             token_price = group.token_price_from_cache(cache, basket_token.token_info.token)
 
             spot_market: typing.Optional[SpotMarketInfo] = group.spot_markets[index]
-            if spot_market is None:
-                raise Exception(
-                    f"Could not read spot market of token {basket_token.token_info.token.symbol} at index {index} of cache at {cache.address}")
+            init_asset_weight: Decimal
+            init_liab_weight: Decimal
+            if spot_market is not None:
+                init_asset_weight = spot_market.init_asset_weight
+                init_liab_weight = spot_market.init_liab_weight
+            else:
+                perp_market: typing.Optional[PerpMarketInfo] = group.perp_markets[index]
+                if perp_market is None:
+                    raise Exception(
+                        f"Could not read spot or perp market of token {basket_token.token_info.token.symbol} at index {index} of cache at {cache.address}")
+                init_asset_weight = perp_market.init_asset_weight
+                init_liab_weight = perp_market.init_liab_weight
 
             in_orders: Decimal = Decimal(0)
             if basket_token.spot_open_orders is not None and str(basket_token.spot_open_orders) in all_open_orders:
                 open_orders: OpenOrders = all_open_orders[str(basket_token.spot_open_orders)]
                 in_orders = open_orders.quote_token_total + \
-                    (open_orders.base_token_total * token_price.value * spot_market.init_asset_weight)
+                    (open_orders.base_token_total * token_price.value * init_asset_weight)
 
             # Base token calculations:
             #     total_collateral += prices[i] * (init_asset_weights[i] * deposits[i] * deposit_index -  init_liab_weights[i] * borrows[i] * borrow_index)
             # Note: the `AccountBasketToken` in the `Account` already factors the deposit and borrow index.
             weighted: Decimal = in_orders + (token_price.value * ((
-                basket_token.deposit.value * spot_market.init_asset_weight) - (
-                    basket_token.borrow.value * spot_market.init_liab_weight)))
+                basket_token.deposit.value * init_asset_weight) - (
+                    basket_token.borrow.value * init_liab_weight)))
 
             if weighted != 0:
                 collateral_description += [f"{weighted:,.8f} USDC from {basket_token.token_info.token.symbol}"]
